/*
 * E!Kit-1100 boards specific pcmcia routines.
 *
 * Copyright 2003-2005 Device Drivers Limited.
 * Author: kawamoto@devdrv.co.jp
 *
 * This program is largely copies by the Db1x00 source files.
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 * 
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/interrupt.h>
#include <linux/device.h>
#include <linux/init.h>

#include <asm/irq.h>
#include <asm/signal.h>
#include <asm/mach-au1x00/au1000.h>
#include <asm/mach-ekit1100/ekit1100.h>

#include "au1000_generic.h"

#if 0
#define debug(x,args...) printk(KERN_DEBUG "%s: " x, __func__ , ##args)
#else
#define debug(x,args...)
#endif

static BCSR * const bcsr = (BCSR *)BCSR_KSEG1_ADDR;

static unsigned short tps_reg = 0;

#define TPS_DATA_SET  outl(0x20000, SYS_OUTPUTSET)
#define TPS_DATA_CLR  outl(0x20000, SYS_OUTPUTCLR)
#define TPS_CLOCK_SET outl(0x40000, SYS_OUTPUTSET)
#define TPS_CLOCK_CLR outl(0x40000, SYS_OUTPUTCLR)
#define TPS_LATCH_SET outl(0x80000, SYS_OUTPUTSET)
#define TPS_LATCH_CLR outl(0x80000, SYS_OUTPUTCLR)

#define TPS_LENGTH 11

#define TPS_WAKE   0x100
#define TPS_AVCC   0x00C
#define TPS_AVCC0  0x000
#define TPS_AVCC3  0x004
#define TPS_AVCC5  0x008
#define TPS_BVCC   0x0C0
#define TPS_BVCC0  0x000
#define TPS_BVCC3  0x080
#define TPS_BVCC5  0x040

static void tps_write(unsigned short d)
{
	int i;
	
	tps_reg = d;
	
	TPS_DATA_CLR;
	TPS_CLOCK_CLR;
	TPS_LATCH_CLR;
	
	for(i = 0; i < TPS_LENGTH; i++)
	{
		if(d & (1 << (TPS_LENGTH - 1)))
			TPS_DATA_SET;
		else
			TPS_DATA_CLR;
		au_sync_udelay(1);
		
		TPS_CLOCK_SET;
		au_sync_udelay(1);
		TPS_CLOCK_CLR;
		au_sync_udelay(1);
		
		d <<= 1;
	}
	TPS_LATCH_SET;
	au_sync_udelay(1);
	TPS_CLOCK_SET;
	au_sync_udelay(1);
	TPS_CLOCK_CLR;
	au_sync_udelay(1);
	TPS_LATCH_CLR;
	au_sync_udelay(1);
}

static unsigned short tps_read(void)
{
	return tps_reg;
}

struct au1000_pcmcia_socket au1000_pcmcia_socket[PCMCIA_NUM_SOCKS];
extern int au1x00_pcmcia_socket_probe(struct device *, struct pcmcia_low_level *, int, int);

static int ekit1100_pcmcia_hw_init(struct au1000_pcmcia_socket *skt)
{
	skt->irq = skt->nr ? AU1000_GPIO_1 : AU1000_GPIO_0;
	return 0;
}

static void ekit1100_pcmcia_shutdown(struct au1000_pcmcia_socket *skt)
{
	bcsr->cf0 = BCSR_CF0_RST;
	bcsr->cf1 = BCSR_CF1_RST;
	tps_write(TPS_WAKE | TPS_AVCC0 | TPS_BVCC0);
	au_sync_delay(5);
}

static void ekit1100_pcmcia_socket_state(struct au1000_pcmcia_socket *skt, struct pcmcia_state *state)
{
	u32 inserted;
	unsigned char vs;

	state->ready = 0;
	state->vs_Xv = 0;
	state->vs_3v = 0;
	state->detect = 0;

	switch(skt->nr)
	{
	case 0:
		vs = (bcsr->cf0 & BCSR_CF0_VS) >> 2;
		inserted = (bcsr->cf0 & BCSR_CF0_CD);
		break;
	case 1:
		vs = (bcsr->cf1 & BCSR_CF1_VS) >> 2;
		inserted = (bcsr->cf1 & BCSR_CF1_CD);
		break;
	default: /* should never happen */
		return;
	}

	if(inserted) 
		debug("ekit1100 socket %d: inserted %d, vs %d\n", skt->nr, inserted, vs);

	if(inserted)
	{
		switch(vs)
		{
			case 0: /* 5V */
				break;
			case 1: /* 3V */
			case 3:
				state->vs_3v = 1;
				break;
			default:
				/* return without setting 'detect' */
				printk(KERN_ERR "ekit1100 bad VS (%d)\n", vs);
		}
		state->detect = 1;
		state->ready = 1;
	}
	else
	{
		/* if the card was previously inserted and then ejected,
		 * we should turn off power to it
		 */
		if((skt->nr == 0) && (!(bcsr->cf0 & BCSR_CF0_RST)))
		{
			bcsr->cf0 = BCSR_CF0_RST;
			tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC0);
			au_sync_delay(5);
		}
		else if((skt->nr == 1) && (!(bcsr->cf1 & BCSR_CF1_RST)))
		{
			bcsr->cf1 = BCSR_CF1_RST;
			tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC0);
			au_sync_delay(5);
		}
	}

	state->bvd1=1;
	state->bvd2=1;
	state->wrprot=0; 
}

static int ekit1100_pcmcia_configure_socket(struct au1000_pcmcia_socket *skt, struct socket_state_t *state)
{
	int sock = skt->nr;

	debug("config_skt %d Vcc %dV Vpp %dV, reset %d\n", 
			sock, state->Vcc, state->Vpp, 
			state->flags & SS_RESET);

	/* pcmcia reg was set to zero at init time. Be careful when
	 * initializing a socket not to wipe out the settings of the 
	 * other socket.
	 */
	state->Vpp = 0;
	if(sock == 0)
	{
		switch(state->Vcc)
		{
			case 0:  /* Vcc 0.0V */
				tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC0);
				break;
			case 33: /* Vcc 3.3V */
				tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC3);
				break;
			case 50: /* Vcc 5.0V */
				tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC5);
				break;
			default: /* what's this ? */
				tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC0);
				printk(KERN_ERR "%s: bad Vcc %d\n", __FUNCTION__, state->Vcc);
				break;
		}
	}
	else
        {
		switch(state->Vcc)
		{
			case 0:  /* Vcc 0.0V */
				tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC0);
				break;
			case 33: /* Vcc 3.3V */
				tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC3);
				break;
			case 50: /* Vcc 5.0V */
				tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC5);
				break;
			default: /* what's this ? */
				tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC0);
				printk(KERN_ERR "%s: bad Vcc %d\n", __FUNCTION__, state->Vcc);
			break;
		}
	}

	au_sync_delay(300);

	if(sock == 0)
	{
		if(!(state->flags & SS_RESET))
		{
			bcsr->cf0 |=  BCSR_CF0_EN;
			au_sync_delay(300);
			bcsr->cf0 &= ~BCSR_CF0_RST;
			au_sync_delay(100);
		}
		else
		{
			bcsr->cf0 = (bcsr->cf0 & ~BCSR_CF0_EN) | BCSR_CF0_RST;
			au_sync_delay(100);
		}
	}
	else
	{
		if(!(state->flags & SS_RESET))
		{
			bcsr->cf1 |=  BCSR_CF1_EN;
			au_sync_delay(300);
			bcsr->cf1 &= ~BCSR_CF1_RST;
			au_sync_delay(100);
		}
		else
		{
			bcsr->cf1 = (bcsr->cf1 & ~BCSR_CF1_EN) | BCSR_CF1_RST;
			au_sync_delay(100);
		}
	}

	return 0;
}

/*
 * Enable card status IRQs on (re-)initialisation.  This can
 * be called at initialisation, power management event, or
 * pcmcia event.
 */
void ekit1100_socket_init(struct au1000_pcmcia_socket *skt)
{
	/* nothing to do for now */
}

/*
 * Disable card status IRQs and PCMCIA bus on suspend.
 */
void ekit1100_socket_suspend(struct au1000_pcmcia_socket *skt)
{
	/* nothing to do for now */
}

struct pcmcia_low_level ekit1100_pcmcia_ops =
{ 
	.owner            = THIS_MODULE,
	.hw_init          = ekit1100_pcmcia_hw_init,
	.hw_shutdown      = ekit1100_pcmcia_shutdown,
	.socket_state     = ekit1100_pcmcia_socket_state,
	.configure_socket = ekit1100_pcmcia_configure_socket,
	.socket_init      = ekit1100_socket_init,
	.socket_suspend   = ekit1100_socket_suspend
};

int __init au1x_board_init(struct device *dev)
{
	int ret = -ENODEV;
	bcsr->cf0 = BCSR_CF0_RST;
	bcsr->cf1 = BCSR_CF1_RST;
	tps_write(TPS_WAKE | TPS_AVCC0 | TPS_BVCC0);
	au_sync_delay(5);
	ret = au1x00_pcmcia_socket_probe(dev, &ekit1100_pcmcia_ops, 0, 2);
	return ret;
}
