/*
 * E!Kit-1100 Boards
 *
 * Copyright 2003-2005 Device Drivers Limited.
 * Author: kawamoto@devdrv.co.jp
 *
 * This program is largely copies by the Db1x00 source files.
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 * 
 */

#ifndef __ASM_EKIT1100_H
#define __ASM_EKIT1100_H

#include <linux/config.h>

#define BCSR_KSEG1_ADDR 0xB8000000

/*
 * Overlay data structure of the E!Kit-1100 board registers.
 * Registers located at physical 0x180000xx, KSEG1 0xB80000xx
 */
typedef volatile struct
{
	/*00*/ unsigned long cf0;
	/*04*/ unsigned long cf1;
	/*08*/ unsigned long usb0;
	/*0C*/ unsigned long usb1;
	/*10*/ unsigned long sys;
	/*14*/ unsigned long gpio3;
	/*18*/ unsigned long rsv;
	/*1C*/ unsigned long config;
} BCSR;


/*
 * Register/mask bit definitions for the BCSRs
 */
#define BCSR_CF0_EN			0x00800000
#define BCSR_CF0_RST			0x00400000
#define BCSR_CF0_IRQ			0x00000020
#define BCSR_CF0_CHG			0x00000010
#define BCSR_CF0_VS			0x0000000C
#define BCSR_CF0_CD			0x00000003

#define BCSR_CF1_EN			0x00800000
#define BCSR_CF1_RST			0x00400000
#define BCSR_CF1_IRQ			0x00000020
#define BCSR_CF1_CHG			0x00000010
#define BCSR_CF1_VS			0x0000000C
#define BCSR_CF1_CD			0x00000003

#define BCSR_USB0_OC1			0x00000080
#define BCSR_USB0_OC0			0x00000040
#define BCSR_USB0_USBID			0x00000010
#define BCSR_USB0_USBAVV		0x00000008
#define BCSR_USB0_USBASV		0x00000004
#define BCSR_USB0_USBBSV		0x00000002
#define BCSR_USB0_USBBSE		0x00000001

#define BCSR_USB1_EN1			0x00000080
#define BCSR_USB1_EN0			0x00000040
#define BCSR_USB1_USBPUV		0x00000020
#define BCSR_USB1_USBPDV		0x00000010
#define BCSR_USB1_USBPDM		0x00000004
#define BCSR_USB1_USBPUP		0x00000002
#define BCSR_USB1_USBPDP		0x00000001

#define BCSR_SYS_PSTAT			0x00800000
#define BCSR_SYS_POFF			0x00200000
#define BCSR_SYS_RPHY0			0x00000008
#define BCSR_SYS_R232C			0x00000004
#define BCSR_SYS_RCF			0x00000002
#define BCSR_SYS_REXT			0x00000001

#define BCSR_GPIO3_OUT			0x003F0000
#define BCSR_GPIO3_P			0x0000003F

#define BCSR_CONFIG_REV			0x00FF0000
#define BCSR_CONFIG_ROSIZ		0x000000F0
#define BCSR_CONFIG_RASIZ		0x0000000C
#define BCSR_CONFIG_RANUM		0x00000002
#define BCSR_CONFIG_ENDIAN		0x00000001

/* PCMCIA E!Kit-1100 specific defines */
#define PCMCIA_MAX_SOCK 1
#define PCMCIA_NUM_SOCKS (PCMCIA_MAX_SOCK+1)

/* SD controller macros */
/*
 * Detect card.
 */
#define mmc_card_inserted(_n_, _res_) \
	do { \
		unsigned long mmc_cd, board_specific; \
		if ((_n_)) { \
			mmc_cd = 1 << 31; /* GPIO31 */ \
		} else { \
			mmc_cd = 1 << 30; /* GPIO30 */ \
		} \
		board_specific = au_readl(SYS_PINSTATERD); \
		if (!(board_specific & mmc_cd)) {/* low means card present */ \
			*(int *)(_res_) = 1; \
		} else { \
			*(int *)(_res_) = 0; \
		} \
	} while (0)

/*
 * Apply power to card slot(s).
 */
#define mmc_power_on(_n_) \
	do { \
		unsigned long mmc_pwr, mmc_cd, board_specific; \
		if ((_n_)) { \
			mmc_pwr = 1 << 23; /* GPIO23 */ \
			mmc_cd =  1 << 31; /* GPIO31 */ \
		} else { \
			mmc_pwr = 1 << 22; /* GPIO22 */ \
			mmc_cd =  1 << 30; /* GPIO30 */ \
		} \
		board_specific = au_readl(SYS_PINSTATERD); \
		if (!(board_specific & mmc_cd)) {/* low means card present */ \
			au_writel(mmc_pwr, SYS_OUTPUTCLR); \
			au_sync(); \
		} \
	} while (0)


#endif /* __ASM_EKIT1100_H */
